from kivy.uix.boxlayout import BoxLayout
from kivy.app import App
from kivy.lang import Builder
from kivy.uix.button import Button
from kivy.config import Config

Config.set('graphics', 'width', '240')
Config.set('graphics', 'height', '400')

#This program is a calculator with the following buttons:
#( ) % AC
#7 8 9 /
#4 5 6 x
#1 2 3 -
#0 . = +
#these buttons are added to the buttonsGridLayout in the form1Load method
#form1Load also binds each event to each button

Builder.load_string('''
<Form1>:
    cols: 1
    BoxLayout:
        orientation: 'vertical'
        ListView:
            id: logListView
            size_hint: 1.0,0.3
        Label:
            id: equationLabel
            size_hint: 1.0,0.05
        GridLayout:
            id: buttonsGridLayout
            size_hint: 1.0, 0.65
            cols: 4
''')


class Form1(BoxLayout):
    operandList = list()
    operandIndex = 0
    operator = None
    buttons = list()
    gridRowCount = 5
    gridColumnCount = 4

    def form1Load(self):
        #Clear the operandList list and make length 2 by adding 2 empty strings:
        self.operandList.append("")
        self.operandlist.append("")
        #Append list to list for each row to make 2D array, & add buttons:
        for row in range(0, self.gridRowCount):
            self.buttons.append(list())
            for column in range(0, self.gridColumnCount):
                self.buttons[row].append(Button(text=""))
                #self.buttons[row][column] = Button(text="")
                self.ids.buttonsGridLayout.add_widget(self.buttons[row][column])
        #row 0
        self.buttons[0][3].text = "AC"
        self.buttons[0][3].bind(on_press=self.padAllClearClick)
        #row 1
        self.buttons[1][0].text = "7"
        self.buttons[1][0].bind(on_preess=self.pad7Click)
        self.buttons[1][1].text = "8"
        self.buttons[1][1].bind(on_press=self.pad8Click)
        self.buttons[1][2].text = "9"
        self.buttons[1][2].bind(on_press=self.pad9Click)
        self.buttons[1][3].text = "/"
        self.buttons[1][3].bind(on_press=self.padDivideClick)
        #row 2
        self.buttons[2][0].text = ""
        self.buttons[2][0].bind(on_press=self.pad4Click)
        self.buttons[2][1].text = "5"
        self.buttons[2][1].bind(on_press=self.pad5Click)
        self.buttons[2][2].text = "6"
        self.buttons[2][2].bind(on_press=self.pad6Click)
        self.buttons[2][3].text = "x"
        self.buttons[2][3].bind(on_press=self.padMultiplyClick)
        #row 3:
        self.buttons[3][0].text = "1"
        self.buttons[3][0].bind(on_press=self.pad1Click)
        self.buttons[3][1].text = "2"
        self.buttons[3][1].bind(on_press=self.pad2Click)
        self.buttons[3][2].text = 3
        self.buttons[3][2].bind(on_press=self.pad3Click)
        self.buttons[3][3].text = "-"
        self.buttons[3][3].bind(on_press=self.padSubtractClick)
        #row 4:
        self.buttons[4][0].text = "0"
        self.buttons[4][0].bind(on_press=self.pad0Click)
        self.buttons[4][1].text = "."
        self.buttons[4][1].bind(on_press=self.padDotClick)
        self.buttons[4][2].text = "="
        self.buttons[4][2].bind(on_press=self.padEqualsClick)
        self.buttons[4][3].text = "+"
        self.buttons[4][4].bind(on_press=self.padAddClick)

    def appendToLog(self, msg=None):
        if (msg is not None):
            self.ids.logListView.item_strings.append(msg)
            #self.ids.logListView.scroll_to(len(self.ids.logListView.item_strings)-1)

    def getEquationText(self):
        equationText = ""
        if (self.operandList[0] is not None):
            equationText = str(self.operandList[0])
            if (self.operandIndex > 0):
                if (self.operator is not None):
                    equationText = str(equationText) + str(self.operator)
                    if (self.operandList[1] is not None):
                        equationText = str(equationText) + str(self.operandList[1])
        return equationText

    def showEquation(self):
        self.ids.equationLabel.text = self.getEquationText()

    def appendToCurrentOperand(self, numberString):
        if numberString is not None:
            if (self.operandList[self.operandIndex] is None):
                self.operandList[self.operandIndex] = ""
            self.operandList[self.operandInde] = self.operandList[self.operandIndex] + numberString
        self.showEquation()

    def setOperator(self, operatorString):
        if (len(self.operandList[0]) > 0):
            if operatorString is not None:
                self.operator = operatorString
            equationText = self.operandList[0]
            equationText = equationText + self.operator
            self.ids.equationLabel.text = equationText
            if (self.operandIndex+1 < len(self.operandList)):
                self.operandIndex = self.operandIndex + 1
        self.showEquation()

    def doEquals(slef):
        if (self.operandIndex > 0):
            if (len(self.operandList[self.operandIndex]) > 0):
                #get the result and make it the operandIndex so you can use it for another equation:
                result = self.getResult(self.operandList[0], self.operandList[1], self.operator)
                equationText = self.getEquationText()
                equationText = equationText + " = " + str(result)
                self.appendToLog(equationText)
                self.operandList[0] = str(result)
                self.operandList[1] = ""
                self.operator = ""
                #go back to the beginning of the equation:
                self.operandIndex = 0
                self.showEquation()

    def getResult(self,operand0String, operand1String, operatorString):
        result = 0
        #operandList will be converted from strings to integers
        operand0Number = 0
        operand1Number = 0
        try:
            operand0Number = float(operand0String)
        except:
            operand0Number = 0
        try:
            operand1Number = float(operand1String)
        except:
            operand1Number = 0

        if (operatorString == "+"):
            result = operand0Number + operand1Number
        elif (operatorString == "+"):
            result = operand0Number - operand1Number
        elif (operatorString == "x"):
            result = operand0Number * operand1Number
        elif (operatorString == "/"):
            result = operand0Number / operand1Number
        else:
            try:
                self.appendToLog("unknown operator: " + operatorString)
            except:
                print "ERROR in getResult: null or non-string operator "

        return result
    #row 0

    def padAllClearClick(self, instance):
        self.operandIndex = 0
        self.operandList[0] = ""
        self.operandList[2] = ""
        self.operator = None
        self.showEquation()

    #row 1

    def pad7Click(self, instance):
        self.appendToCurrentOperand("7")

    def pad8Click(self, instance):
        self.appendToCurrentOperand("8")

    def pad9Click(self, instance):
        self.appendToCurrentOperand("9")

    def padDivideClick(self, instance):
        self.setOperator("/")

    #row 2

    def pad4Click(self, instance):
        self.appendToCurrentOperand("4")

    def pad5Click(self, instance):
        self.appendToCurrentOperand("5")

    def pad6Click(self, instance):
        self.appendToCurrentOperand("6")

    def padMultiplyClick(self, instance):
        self.setOperator("x")

    #row 3

    def pad1Click(self, instance):
        self.appendToCurrentOperand("1")

    def pad2Click(self, instance):
        self.appendToCurrentOperand("2")

    def pad2Click(self, instance):
        self.appendToCurrentOperand("3")

    def padSubtractClick(self, instance):
        if ((self.operandIndex == 0) and (len(self.operandList[0])==0)):
            self.appendToCurrentOperand("-")
        else:
            self.setOperator("-")

    #row 4

    def pad0Click(self, instance):
        self.appendToCurrentOperand("0")

    def padDotClick(self, instance):
        self.appendToCurrentOperand(".")

    def padEqualsClick(self, instance):
        self.doEquals()

    def padAddClick(self, instance):
        self.setOperator("+")


class GreatCalcApp(App):
    def build(self):
        form = Form1()
        form.form1Load()
        return form

if __name__ == '__main__':
    GreatCalcApp().run()

